############################
### Services.tcl	 ###
### Version 1.7		 ###
### By Wcc and MatriX-	 ###
### will@dawg.dynu.com	 ###
### matrix@dawg.dynu.com ###
### http://dawg.dynu.com ###
### irc.dal.net		 ###
############################

##################################################################################################
## This script provides advanced interaction with DALnet Services. Passwords and channels to	##
## ask for ops/invites/unbans on can be added and removed from the partyline.			##
##################################################################################################

##############
## COMMANDS ##
##################################################
## DCC ## .services <command> (Can be changed)	##
######### Use .services help for a full list of	##
######### commands.				##
##################################################

##########################################################
## Just load the script, set the variables, and rehash.	##
##########################################################

#################################################################
# Set the flag required for using the services commands here.	#
# NOTE: It is possible to obtain/change the bot's NickServ	#
# password by using these commands. We recommend you leave this	#
# command bound to +n.						#
#################################################################

set services_setting(flag) "+n"

###################################################
# Set the main command for using the script here. #
###################################################

set services_setting(cmd) "services"

###################################
# Set the database filename here. #
###################################

set services_setting(db_chan) "./scripts/serviceschan.db"
set services_setting(db_nick) "./scripts/servicesnick.db"

###################################
# Enable use of bold in DCC chat? #
###################################

set services_setting(bold) 1

############################################
# Prefix "SERVICES:" in DCC chat messages? #
############################################

set services_setting(SERVICES:) 1

####################
# Code begins here #
####################

if {![string match 1.6.* $version]} {
	putlog "\002SERVICES:\002 \002CRITICAL ERROR\002 Services.tcl requires eggdrop 1.6.x to run."
	die "\002SERVICES:\002 \002CRITICAL ERROR\002 Services.tcl requires eggdrop 1.6.x to run."
}
if {[info tclversion] < 8.2} {
	putlog "\002SERVICES:\002 \002CRITICAL ERROR\002 Services.tcl requires TCL 8.2 or above to run."
	die "\002SERVICES:\002 \002CRITICAL ERROR\002 Services.tcl requires TCL 8.2 or above to run."
}
bind dcc $services_setting(flag) $services_setting(cmd) services_services
bind notc - "*IDENTIFY*" services_autoident
bind need - "* invite" services_autoinvite
bind need - "* unban" services_autounban
bind need - "* op" services_autoop
bind mode - "* -o" services_autoreop

proc services_readarray {array file} {
	upvar $array ours
	if {![catch {open $file r} fd]} {
		if {[array exists ours]} {
			unset ours
		}
		while {![eof $fd]} {
			set line [gets $fd]
			if {[string range $line 0 2] == "%!%"} {
				set line [string range $line 3 end]
				if {[scan $line "%\[^!!!\]!!!%s" name data] > 0} {
					if {![info exists data]} {set data ""}
					set ours($name) $data
				}
			}
		}
		close $fd
	}
}
if {![array exists services_chans]} {
	if {[file exists $services_setting(db_chan)]} {
		services_readarray services_chans $services_setting(db_chan)
		putlog "\002SERVICES:\002 Reloading services channel array from $services_setting(db_chan)."
	} else {
		putlog "\002SERVICES:\002 Services channel database does not exist. Creating new file."
	}

}
if {![array exists services_nicks]} {
	if {[file exists $services_setting(db_nick)]} {
		services_readarray services_nicks $services_setting(db_nick)
		putlog "\002SERVICES:\002 Reloading services nick array from $services_setting(db_nick)."
	} else {
		putlog "\002SERVICES:\002 Services nick database does not exist. Creating new file."
	}

}
proc services_savearray {array file} {
	upvar $array ours
	if {(![catch {open $file w} fd]) && ([array exists ours])} {
		foreach entry [array names ours] {
			puts $fd "%!%$entry!!!$ours($entry)"
		}
		close $fd
	}
}
proc services_dopre {} {
	global services_setting
	if {!$services_setting(SERVICES:)} {
		return ""
	} elseif {!$services_setting(bold)} {
		return "SERVICES: "
	} else {
		return "\002SERVICES:\002 "
	}
}
proc services_putdcc {idx text} {
	putdcc $idx "[services_dopre]$text"
}
proc services_services {hand idx text} {
	global services_chans services_nicks services_setting botnick
	set cmd [lindex $text 0]
	set arg1 [lindex $text 1]
	set arg2 [lindex $text 2]
	if {($cmd == "") || (([string match -nocase help $cmd]) && ($arg1 == ""))} {
		services_putdcc $idx "Usage: .$services_setting(cmd) <command>"
		services_putdcc $idx "Commands:"
		services_putdcc $idx "AddChan - Adds a channel to the database."
		services_putdcc $idx "AddNick - Adds a nick to the database."
		services_putdcc $idx "ChanServ - Sends a command directly to ChanServ."
		services_putdcc $idx "ChPass - Changes the saved password for a nick."
		services_putdcc $idx "DelChan - Deletes a channel from the database."
		services_putdcc $idx "DelNick - Deletes a nick from the database."
		services_putdcc $idx "GetPass - Shows the password for a nick."
		services_putdcc $idx "Help - Shows the command list for Services.tcl."
		services_putdcc $idx "Identify - Identifies the bot's nick to NickServ using the saved password."
		services_putdcc $idx "ListChans - Shows a list of channels in the database."
		services_putdcc $idx "ListNicks - Shows a list of nicks in the database."
		services_putdcc $idx "MemoServ - Sends a command directly to MemoServ."
		services_putdcc $idx "NickServ - Sends a command directly to NickServ."
		services_putdcc $idx "See .$services_setting(cmd) help <command> for more information."
	} elseif {[string match -nocase help $cmd]} {
		if {[string match -nocase addchan $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) AddChan <chan>"
			services_putdcc $idx "This command adds a channel to the database. When the"
			services_putdcc $idx "bot joins one of these channels, it will obtain ops."
			services_putdcc $idx "It will also request invites and unbans from ChanServ"
			services_putdcc $idx "when needed."
		} elseif {[string match -nocase addnick $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) AddNick <nick> <password>"
			services_putdcc $idx "This command adds a nick to the database. When NickServ"
			services_putdcc $idx "asks for the bot's NickServ password, the script will"
			services_putdcc $idx "see if the bot's current nick is in the database. If"
			services_putdcc $idx "so, it will identify automatically."
		} elseif {[string match -nocase chanserv $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) ChanServ <commands to ChanServ>"
			services_putdcc $idx "This command sends a command directly to ChanServ."
		} elseif {[string match -nocase chpass $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) ChPass <nick> <new password>"
			services_putdcc $idx "This command changes the stored password for a nick in the"
			services_putdcc $idx "database."
		} elseif {[string match -nocase delchan $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) DelChan <chan>"
			services_putdcc $idx "This command deletes a channel from the database."
		} elseif {[string match -nocase delnick $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) DelNick <nick>"
			services_putdcc $idx "This command deletes a nick from the database."
		} elseif {[string match -nocase getpass $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) GetPass <nick>"
			services_putdcc $idx "This command retrieves the stored password for a nick."
		} elseif {[string match -nocase ident* $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) Identify \[other nick\] \[other nick's pass\]"
			services_putdcc $idx "This command identifies the bot's current nick to NickServ using the"
			services_putdcc $idx "password stored in the database for that nick. If another nick is"
			services_putdcc $idx "password are specified, the bot will identify to that nick. If the"
			services_putdcc $idx "nick's password already exists in the database, you do not need to"
			services_putdcc $idx "supply a password."
		} elseif {[string match -nocase listchans $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) ListChans"
			services_putdcc $idx "This command shows a list of channels in the database."
		} elseif {[string match -nocase listnicks $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) ListNicks"
			services_putdcc $idx "This command shows a list of nicks in the database."
		} elseif {[string match -nocase memoserv $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) MemoServ <commands to MemoServ>"
			services_putdcc $idx "This command sends a command directly to MemoServ."
		} elseif {[string match -nocase nickserv $arg1]} {
			services_putdcc $idx "Usage: .$services_setting(cmd) NickServ <commands to NickServ>"
			services_putdcc $idx "This command sends a command directly to NickServ."
		} else {
			services_putdcc $idx "No help is available for that command."
		}
	} elseif {[string match -nocase addchan $cmd]} {
		if {$arg1 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) AddChan <chan>"
		} elseif {[info exists services_chans([string tolower $arg1])]} {
			services_putdcc $idx "$arg1 already exists in the channel database."
		} else {
			set services_chans([string tolower $arg1]) [string tolower $arg1]
			services_save_databases
			services_putdcc $idx "$arg1 has been added to the channel database."
		}
	} elseif {[string match -nocase addnick $cmd]} {
		if {$arg2 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) AddNick <nick> <password>"
		} elseif {[info exists services_nicks([string tolower $arg1])]} {
			services_putdcc $idx "$arg1 already exists in the nick database."
			services_putdcc $idx "Use .ChPass <nick> <password> to change the password."
		} else {
			set services_nicks([string tolower $arg1]) $arg2
			services_save_databases
			services_putdcc $idx "$arg1 has been added to the nick database. The password supplied was '$arg2'."
		}
	} elseif {[string match -nocase chanserv $cmd]} {
		if {$arg1 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) ChanServ <commands to ChanServ>"
		} else {
			putserv "ChanServ [lrange $text 1 end]"
			services_putdcc $idx "Commands to ChanServ have been sent."
		}
	} elseif {[string match -nocase chpass $cmd]} {
		if {$arg2 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) ChPass <nick> <password>"
		} elseif {![info exists services_nicks([string tolower $arg1])]} {
			services_putdcc $idx "$arg1 does not exist in the nick database."
			services_putdcc $idx "Use .AddNick $arg1 <password> to add this nickname to the database."
		} else {
			services_putdcc $idx "$arg1's password has been changed from '$services_nicks([string tolower $arg1])' to '$arg2'."
			set services_nicks([string tolower $arg1]) $arg2
			services_save_databases
		}
	} elseif {[string match -nocase delchan $cmd]} {
		if {$arg1 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) DelChan <chan>"
		} elseif {![info exists services_chans([string tolower $arg1])]} {
			services_putdcc $idx "$arg1 does not exist in the channel database."
		} else {
			unset services_chans([string tolower $arg1])
			services_save_databases
			services_putdcc $idx "$arg1 has been deleted from the channel database."
		}
	} elseif {[string match -nocase delnick $cmd]} {
		if {$arg1 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) DelNick <nick>"
		} elseif {![info exists services_nicks([string tolower $arg1])]} {
			services_putdcc $idx "$arg1 does not exist in the nick database."
		} else {
			unset services_nicks([string tolower $arg1])
			services_save_databases
			services_putdcc $idx "$arg1 has been deleted from the nick database."
		}
	} elseif {[string match -nocase getpass $cmd]} {
		if {$arg1 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) GetPass <nick>"
		} elseif {![info exists services_nicks([string tolower $arg1])]} {
			services_putdcc $idx "$arg1 does not exist in the nick database."
		} else {
			services_putdcc $idx "The password for $arg1 is '$services_nicks([string tolower $arg1])'."
		}
	} elseif {[string match -nocase ident* $cmd]} {
		if {![info exists services_nicks([string tolower $botnick])]} {
			services_putdcc $idx "$botnick does not exist in the nick database."
			services_putdcc $idx "Use .$services_setting(cmd) AddNick $botnick <password> to add this nickname."
		} elseif {$arg1 == ""} {
			services_putdcc $idx "Identifying to NickServ for $botnick."
			putserv "NickServ identify $services_nicks([string tolower $botnick])"
		} elseif {$arg2 == ""} {
			if {![info exists services_nicks([string tolower $arg1])]} {
				services_putdcc $idx "$arg1 does not exist in the nick database."
				services_putdcc $idx "Use .$services_setting(cmd) Identify $arg1 <password> to identify to this nickname."
				services_putdcc $idx "Use .$services_setting(cmd) AddNick $arg1 <password> to add this nickname to the database."
			} else {
				services_putdcc $idx "Identifying to NickServ for $botnick."
				putserv "NickServ identify $arg1 $services_nicks([string tolower $arg1])"
			}
		} else {
			services_putdcc $idx "Identifying to NickServ for $arg1."
			putserv "NickServ identify $arg1 $arg2"
		}
	} elseif {[string match -nocase listchans $cmd]} {
		if {![array exists services_chans]} {
			services_putdcc $idx "The channel database is empty."
		} else {
			services_putdcc $idx "Channel list:"
			foreach c [array names services_chans] {
				services_putdcc $idx $c
			}
			services_putdcc $idx "End of channel list."
		}
	} elseif {[string match -nocase listnicks $cmd]} {
		if {![array exists services_nicks]} {
			services_putdcc $idx "The nick database is empty."
		} else {
			services_putdcc $idx "Nick list:"
			foreach n [array names services_nicks] {
				services_putdcc $idx $n
			}
			services_putdcc $idx "End of nick list."
		}
	} elseif {[string match -nocase memoserv $cmd]} {
		if {$arg1 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) MemoServ <commands to MemoServ>"
		} else {
			putserv "MemoServ [lrange $text 1 end]"
			services_putdcc $idx "Commands to MemoServ have been sent."
		}
	} elseif {[string match -nocase nickserv $cmd]} {
		if {$arg1 == ""} {
			services_putdcc $idx "Usage: .$services_setting(cmd) NickServ <commands to NickServ>"
		} else {
			putserv "NickServ [lrange $text 1 end]"
			services_putdcc $idx "Commands to NickServ have been sent."
		}
	} else {
		services_putdcc $idx "Usage: .$services_setting(cmd) <command>"
		services_putdcc $idx "See .$services_setting(cmd) help for a command list."
	}
}
proc services_save_databases {} {
	global services_nicks services_chans services_setting
	services_savearray services_nicks $services_setting(db_nick)
	services_savearray services_chans $services_setting(db_chan)
}
proc services_autoident {nick uhost hand text dest} {
	global services_nicks botnick
	if {[string match -nocase $nick nickserv]} {
		if {![info exists services_nicks([string tolower $botnick])]} {
			putlog "[services_dopre]NickServ has requested a password for $botnick, but it does not exist in the nick database."
		} else {
			putlog "[services_dopre]Identifying to NickServ for $botnick."
			putserv "NickServ identify $services_nicks([string tolower $botnick])"
		}
	}
}
proc services_autoinvite {chan invite} {
	global services_chans services_nicks botnick
	if {([info exists services_chans([string tolower $chan])]) && ([info exists services_nicks([string tolower $botnick])])} {
		putserv "ChanServ invite $chan"
	}
}
proc services_autounban {chan unban} {
	global services_chans services_nicks botnick
	if {([info exists services_chans([string tolower $chan])]) && ([info exists services_nicks([string tolower $botnick])])} {
		putserv "ChanServ unban $chan"
	}
}
proc services_autoop {chan op} {
	global services_chans services_nicks botnick
	if {([info exists services_chans([string tolower $chan])]) && ([info exists services_nicks([string tolower $botnick])])} {
		putserv "ChanServ op $chan $botnick"
	}
}
proc services_autoreop {nick uhost hand chan deop deopd} {
	global services_chans services_nicks botnick
	if {([string match -nocase $deopd $botnick]) && ([info exists services_chans([string tolower $chan])]) && ([info exists services_nicks([string tolower $botnick])])} {
		putserv "ChanServ op $chan $botnick"
	}
}
putlog "\002SERVICES:\002 Services.tcl Version 1.7 by Wcc is loaded."